;;**********************************************************************************************
;       16x16 Bit Signed Fixed Point Multiply 16 x 16 -> 32
;       Input:  16 bit signed fixed point multiplicand in AARGB0, AARGB1
;               16 bit signed fixed point multiplier in BARGB0, BARGB1
;       Use:    CALL    FXM1616S
;       Output: 32 bit signed fixed point product in AARGB0, AARGB1,
;               AARGB2, AARGB3
;       Result: AARG  <--  AARG * BARG
;       Max Timing:     39 clks
;       Min Timing:     31 clks
;       PM: 38              DM: 8
;prOG	CODE

FXM1608S
;     - , 
;       -    
;   
;    S16*U8  .
;		GLOBAL	FXM1616S		;
;Tmp_0,BANKED	ARG_B,BANKED	,BANKED
		MOVFF16	_A,Tmp_0		;	   

;1)   :
		MOVF	__AARGB1,W		;   
		MULWF	__BARGB1		;     
		MOVFF	PRODH,__AARGB2	;    16  
		MOVFF	PRODL,__AARGB3	;   
		
;		MOVF	__AARGB0,W		;    
;		MULWF	__BARGB0		;   =0

		MOVFF	PRODH,__AARGB0	; 			; clrf
		MOVFF	PRODL,__AARGB1	;   	; clrf

		MULWF	__BARGB1		;       
 
		MOVF	PRODL,W			;  16  
		ADDWF	__AARGB2,F		;  8*8

		MOVF	PRODH,W			;   32 
		ADDWFC	__AARGB1,F		;   

		CLRF	WREG			; ADD16 Macro
		ADDWFC	__AARGB0,F		;

;		MOVF	__TEMPB1,W		;
;		MULWF	__BARGB0		; = 0

;		MOVF	PRODL,W			;
;		ADDWF	__AARGB2,F		;

;		MOVF	PRODH,W			;
;		ADDWFC	__AARGB1,F		;

;		CLRF	WREG			;  
;		ADDWFC	__AARGB0,F		;    

;		BTFSS	__BARGB0,MSB	;     ?
;		GOTO	TSIGN1616A		; , 

;		MOVF	__TEMPB1,W		; ,   
;		SUBWF	__AARGB1,F		;  
;		MOVF	__TEMPB0,W		;   
;		SUBWFB	__AARGB0,F		; -

TSIGN1616A
		BTFSS	__TEMPB0,MSB	;     ?
		RETLW	0x00			; ,  

		MOVF	__BARGB1,W		; ,    
		SUBWF	__AARGB1,F		;  
		MOVF	__BARGB0,W		;   
		SUBWFB	__AARGB0,F		; -
		
		RETLW	0x00			;


;**********************************************************************************************
;FXM16S08U		macro		ArgA,A1A,Lit16 	;    -  
;   16    (-) -   16  
; (  ),      8  
;      .
; 24      11  
;
;    FXM1616U    
;**********************************************************************************************
;       -18 \src\traditional\math\fxm1616s.asm
;    1616=32     168.
;       
;Equation 6-1: 16 x 16 Unsigned Multiplication Algorithm
;RES3:RES2:RES1:RES0 = ARG1H:ARG1L  ARG2H:ARG2L
;= (ARG1H  ARG2H  2^16)+	;   
;(ARG1H  ARG2L  2^8)+		;    
;(ARG1L  ARG2H  2^8)+		;       
;(ARG1L  ARG2L)			;     
;   .
;MOVFF ARG1L, WREG
;MULWF ARG2L ; ARG1L * ARG2L ->
;; PRODH:PRODL
;MOVFF PRODH, RES1 ;
;MOVFF PRODL, RES0 ;
;MOVFF ARG1H, WREG
;MULWF ARG2H ; ARG1H * ARG2H ->
;; PRODH:PRODL
;MOVFF PRODH, RES3 ;
;MOVFF PRODL, RES2 ;
;MOVFF ARG1L, WREG
;MULWF ARG2H ; ARG1L * ARG2H ->
;; PRODH:PRODL
;MOVFF PRODL, WREG ;
;ADDWF RES1, F ; Add cross
;MOVFF PRODH, WREG ; products
;ADDWFC RES2, F ;
;CLRF WREG, F ;
;ADDWFC RES3, F ;
;MOVFF ARG1H, WREG ;
;MULWF ARG2L ; ARG1H * ARG2L ->
;; PRODH:PRODL
;MOVFF PRODL, WREG ;
;ADDWF RES1, F ; Add cross
;MOVFF PRODH, WREG ; products
;ADDWFC RES2, F ;
;CLRF WREG, F ;
;ADDWFC RES3, F ;
;

;    16     16     16   - 
;     .
;       16x16 Bit Unsigned Fixed Point Multiply 16 x 16 -> 32
;       Input:  16 bit unsigned fixed point multiplicand in AARGB0(High), AARGB1(Low)
;               16 bit unsigned fixed point multiplier in BARGB0(High), BARGB1(Low)
;       Use:    CALL    FXM1616U
;       Output: 32 bit unsigned fixed point product in AARGB0, AARGB1, 
;               AARGB2, AARGB3
;       Result: AARG  <--  AARG * BARG

; RES3:RES0 = ARG1H:ARG1L  ARG2H:ARG2L =
;  (ARG1H  ARG2H  2^16) +
;  (ARG1H  ARG2L  2^8)  +
;  (ARG1L  ARG2H  2^8)  +
;  (ARG1L  ARG2L  2^0)

; ArgA  ArgB     -   -  
;    16   (+)  16  ,
;     16 .
;       (-) +   +2 (-)

; MOVFF	ArgA+0x0,_Tmp+0x0	;MOVFF	__AARGB1,__TEMPB1	;    
; MOVFF	ArgA+0x1,_Tmp+0x1	;MOVFF	__AARGB1,__TEMPB1	;    
;
; MOVLW	LOW Lit16			;MOVF	__AARGB1,W			; 
; MULWF	ArgA+0x0			;MULWF	__BARGB1			; 
; MOVFF	PRODH,_Tmp+0x3		;MOVFF	PRODH,__AARGB2		; 
; ;   -  - ,    
; ;MOVFF	PRODL,_Tmp+0x2		;MOVFF	PRODL,__AARGB3		; ,  -    
;		
; MOVLW	HIGH Lit16			;MOVF	__AARGB0,W			;  
; MULWF	ArgA+0x1,BANKED		;MULWF	__BARGB0			; 
; MOVFF	PRODH,ArgA+0x1		;MOVFF	PRODH,__AARGB0		; 
; MOVFF	PRODL,ArgA+0x0		;MOVFF	PRODL,__AARGB1		;  -   
;
; MULWF	_Tmp+0x0,BANKED		;MULWF	__BARGB1			;  L-16   -16 
;
; MOVF	PRODL,W,ACCESS							;    
; ADDWF	_Tmp+0x3,F,BANKED	;ADDWF	__AARGB2,F	; 
; MOVF	PRODH,W,ACCESS							; 
; ADDWFC	ArgA+0x0,F,BANKED	;ADDWFC	__AARGB1,F	;  
; CLRF	WREG,ACCESS								;  1 
; ADDWFC	ArgA+0x1,F,BANKED	;ADDWFC	__AARGB0,F	;   
;
; MOVLW	LOW Lit16			; MOVF	__TEMPB1,W	;  L-16 
; MULWF	_Tmp+0x1,BANKED		; MULWF	__BARGB0	;   -16 
;
; MOVF	PRODL,W,ACCESS							;    
; ADDWF	_Tmp+0x3,F,BANKED	;ADDWF	__AARGB2,F	; 
; MOVF	PRODH,W,ACCESS							; 
; ADDWFC	ArgA+0x0,F,BANKED	;ADDWFC	__AARGB1,F	;  
; CLRF	WREG,ACCESS								;  1 
; ADDWFC	ArgA+0x1,F,BANKED	;ADDWFC	__AARGB0,F	;   
;
;		ENDM

;